
from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0x74934378840D77E36AeF1f031D301549b4e1a225",
    "sd": "0x0000000000000000000000000000000000000000"
}

ABI = """[
    {"name": "HeroReroll", "type": "event", "inputs": [{"name": "heroId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "heroBefore", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "summoningInfo", "type": "tuple", "components": [{"name": "summonedTime", "type": "uint256", "internalType": "uint256"}, {"name": "nextSummonTime", "type": "uint256", "internalType": "uint256"}, {"name": "summonerId", "type": "uint256", "internalType": "uint256"}, {"name": "assistantId", "type": "uint256", "internalType": "uint256"}, {"name": "summons", "type": "uint32", "internalType": "uint32"}, {"name": "maxSummons", "type": "uint32", "internalType": "uint32"}], "internalType": "struct IHeroTypes.SummoningInfo"}, {"name": "info", "type": "tuple", "components": [{"name": "statGenes", "type": "uint256", "internalType": "uint256"}, {"name": "visualGenes", "type": "uint256", "internalType": "uint256"}, {"name": "rarity", "type": "uint8", "internalType": "enum IHeroTypes.Rarity"}, {"name": "shiny", "type": "bool", "internalType": "bool"}, {"name": "generation", "type": "uint16", "internalType": "uint16"}, {"name": "firstName", "type": "uint32", "internalType": "uint32"}, {"name": "lastName", "type": "uint32", "internalType": "uint32"}, {"name": "shinyStyle", "type": "uint8", "internalType": "uint8"}, {"name": "class", "type": "uint8", "internalType": "uint8"}, {"name": "subClass", "type": "uint8", "internalType": "uint8"}], "internalType": "struct IHeroTypes.HeroInfo"}, {"name": "state", "type": "tuple", "components": [{"name": "staminaFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "hpFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "mpFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "level", "type": "uint16", "internalType": "uint16"}, {"name": "xp", "type": "uint64", "internalType": "uint64"}, {"name": "currentQuest", "type": "address", "internalType": "address"}, {"name": "sp", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "enum IHeroTypes.HeroStatus"}], "internalType": "struct IHeroTypes.HeroState"}, {"name": "stats", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hp", "type": "uint16", "internalType": "uint16"}, {"name": "mp", "type": "uint16", "internalType": "uint16"}, {"name": "stamina", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroStats"}, {"name": "primaryStatGrowth", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hpSm", "type": "uint16", "internalType": "uint16"}, {"name": "hpRg", "type": "uint16", "internalType": "uint16"}, {"name": "hpLg", "type": "uint16", "internalType": "uint16"}, {"name": "mpSm", "type": "uint16", "internalType": "uint16"}, {"name": "mpRg", "type": "uint16", "internalType": "uint16"}, {"name": "mpLg", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroStatGrowth"}, {"name": "secondaryStatGrowth", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hpSm", "type": "uint16", "internalType": "uint16"}, {"name": "hpRg", "type": "uint16", "internalType": "uint16"}, {"name": "hpLg", "type": "uint16", "internalType": "uint16"}, {"name": "mpSm", "type": "uint16", "internalType": "uint16"}, {"name": "mpRg", "type": "uint16", "internalType": "uint16"}, {"name": "mpLg", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroStatGrowth"}, {"name": "professions", "type": "tuple", "components": [{"name": "mining", "type": "uint16", "internalType": "uint16"}, {"name": "gardening", "type": "uint16", "internalType": "uint16"}, {"name": "foraging", "type": "uint16", "internalType": "uint16"}, {"name": "fishing", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroProfessions"}], "indexed": false, "internalType": "struct IHeroTypes.Hero"}, {"name": "heroAfter", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "summoningInfo", "type": "tuple", "components": [{"name": "summonedTime", "type": "uint256", "internalType": "uint256"}, {"name": "nextSummonTime", "type": "uint256", "internalType": "uint256"}, {"name": "summonerId", "type": "uint256", "internalType": "uint256"}, {"name": "assistantId", "type": "uint256", "internalType": "uint256"}, {"name": "summons", "type": "uint32", "internalType": "uint32"}, {"name": "maxSummons", "type": "uint32", "internalType": "uint32"}], "internalType": "struct IHeroTypes.SummoningInfo"}, {"name": "info", "type": "tuple", "components": [{"name": "statGenes", "type": "uint256", "internalType": "uint256"}, {"name": "visualGenes", "type": "uint256", "internalType": "uint256"}, {"name": "rarity", "type": "uint8", "internalType": "enum IHeroTypes.Rarity"}, {"name": "shiny", "type": "bool", "internalType": "bool"}, {"name": "generation", "type": "uint16", "internalType": "uint16"}, {"name": "firstName", "type": "uint32", "internalType": "uint32"}, {"name": "lastName", "type": "uint32", "internalType": "uint32"}, {"name": "shinyStyle", "type": "uint8", "internalType": "uint8"}, {"name": "class", "type": "uint8", "internalType": "uint8"}, {"name": "subClass", "type": "uint8", "internalType": "uint8"}], "internalType": "struct IHeroTypes.HeroInfo"}, {"name": "state", "type": "tuple", "components": [{"name": "staminaFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "hpFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "mpFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "level", "type": "uint16", "internalType": "uint16"}, {"name": "xp", "type": "uint64", "internalType": "uint64"}, {"name": "currentQuest", "type": "address", "internalType": "address"}, {"name": "sp", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "enum IHeroTypes.HeroStatus"}], "internalType": "struct IHeroTypes.HeroState"}, {"name": "stats", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hp", "type": "uint16", "internalType": "uint16"}, {"name": "mp", "type": "uint16", "internalType": "uint16"}, {"name": "stamina", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroStats"}, {"name": "primaryStatGrowth", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hpSm", "type": "uint16", "internalType": "uint16"}, {"name": "hpRg", "type": "uint16", "internalType": "uint16"}, {"name": "hpLg", "type": "uint16", "internalType": "uint16"}, {"name": "mpSm", "type": "uint16", "internalType": "uint16"}, {"name": "mpRg", "type": "uint16", "internalType": "uint16"}, {"name": "mpLg", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroStatGrowth"}, {"name": "secondaryStatGrowth", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hpSm", "type": "uint16", "internalType": "uint16"}, {"name": "hpRg", "type": "uint16", "internalType": "uint16"}, {"name": "hpLg", "type": "uint16", "internalType": "uint16"}, {"name": "mpSm", "type": "uint16", "internalType": "uint16"}, {"name": "mpRg", "type": "uint16", "internalType": "uint16"}, {"name": "mpLg", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroStatGrowth"}, {"name": "professions", "type": "tuple", "components": [{"name": "mining", "type": "uint16", "internalType": "uint16"}, {"name": "gardening", "type": "uint16", "internalType": "uint16"}, {"name": "foraging", "type": "uint16", "internalType": "uint16"}, {"name": "fishing", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroProfessions"}], "indexed": false, "internalType": "struct IHeroTypes.Hero"}, {"name": "choice", "type": "uint8", "indexed": true, "internalType": "enum Gen0Reroll.RerollChoices"}], "anonymous": false},
    {"name": "Initialized", "type": "event", "inputs": [{"name": "version", "type": "uint8", "indexed": false, "internalType": "uint8"}], "anonymous": false},
    {"name": "RewardMinted", "type": "event", "inputs": [{"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "heroId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "rewardAddress", "type": "address", "indexed": true, "internalType": "address"}, {"name": "amount", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "StatUp", "type": "event", "inputs": [{"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "heroId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "stat", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "increase", "type": "uint8", "indexed": false, "internalType": "uint8"}, {"name": "updateType", "type": "uint8", "indexed": false, "internalType": "uint8"}], "anonymous": false},
    {"name": "SummonsReset", "type": "event", "inputs": [{"name": "heroId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "oldSummonCount", "type": "uint32", "indexed": false, "internalType": "uint32"}, {"name": "newSummonCount", "type": "uint32", "indexed": false, "internalType": "uint32"}], "anonymous": false},
    {"name": "XpUp", "type": "event", "inputs": [{"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "heroId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "increase", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "campaign", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "cancelReroll", "type": "function", "inputs": [{"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "crystals", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "endRoll", "type": "function", "inputs": [{"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "endTime", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "heroToRerollData", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "campaign", "type": "uint256", "internalType": "uint256"}, {"name": "seedblock", "type": "uint256", "internalType": "uint256"}, {"name": "choice", "type": "uint8", "internalType": "enum Gen0Reroll.RerollChoices"}, {"name": "beforeHero", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "summoningInfo", "type": "tuple", "components": [{"name": "summonedTime", "type": "uint256", "internalType": "uint256"}, {"name": "nextSummonTime", "type": "uint256", "internalType": "uint256"}, {"name": "summonerId", "type": "uint256", "internalType": "uint256"}, {"name": "assistantId", "type": "uint256", "internalType": "uint256"}, {"name": "summons", "type": "uint32", "internalType": "uint32"}, {"name": "maxSummons", "type": "uint32", "internalType": "uint32"}], "internalType": "struct IHeroTypes.SummoningInfo"}, {"name": "info", "type": "tuple", "components": [{"name": "statGenes", "type": "uint256", "internalType": "uint256"}, {"name": "visualGenes", "type": "uint256", "internalType": "uint256"}, {"name": "rarity", "type": "uint8", "internalType": "enum IHeroTypes.Rarity"}, {"name": "shiny", "type": "bool", "internalType": "bool"}, {"name": "generation", "type": "uint16", "internalType": "uint16"}, {"name": "firstName", "type": "uint32", "internalType": "uint32"}, {"name": "lastName", "type": "uint32", "internalType": "uint32"}, {"name": "shinyStyle", "type": "uint8", "internalType": "uint8"}, {"name": "class", "type": "uint8", "internalType": "uint8"}, {"name": "subClass", "type": "uint8", "internalType": "uint8"}], "internalType": "struct IHeroTypes.HeroInfo"}, {"name": "state", "type": "tuple", "components": [{"name": "staminaFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "hpFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "mpFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "level", "type": "uint16", "internalType": "uint16"}, {"name": "xp", "type": "uint64", "internalType": "uint64"}, {"name": "currentQuest", "type": "address", "internalType": "address"}, {"name": "sp", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "enum IHeroTypes.HeroStatus"}], "internalType": "struct IHeroTypes.HeroState"}, {"name": "stats", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hp", "type": "uint16", "internalType": "uint16"}, {"name": "mp", "type": "uint16", "internalType": "uint16"}, {"name": "stamina", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroStats"}, {"name": "primaryStatGrowth", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hpSm", "type": "uint16", "internalType": "uint16"}, {"name": "hpRg", "type": "uint16", "internalType": "uint16"}, {"name": "hpLg", "type": "uint16", "internalType": "uint16"}, {"name": "mpSm", "type": "uint16", "internalType": "uint16"}, {"name": "mpRg", "type": "uint16", "internalType": "uint16"}, {"name": "mpLg", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroStatGrowth"}, {"name": "secondaryStatGrowth", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hpSm", "type": "uint16", "internalType": "uint16"}, {"name": "hpRg", "type": "uint16", "internalType": "uint16"}, {"name": "hpLg", "type": "uint16", "internalType": "uint16"}, {"name": "mpSm", "type": "uint16", "internalType": "uint16"}, {"name": "mpRg", "type": "uint16", "internalType": "uint16"}, {"name": "mpLg", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroStatGrowth"}, {"name": "professions", "type": "tuple", "components": [{"name": "mining", "type": "uint16", "internalType": "uint16"}, {"name": "gardening", "type": "uint16", "internalType": "uint16"}, {"name": "foraging", "type": "uint16", "internalType": "uint16"}, {"name": "fishing", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroProfessions"}], "internalType": "struct IHeroTypes.Hero"}, {"name": "status", "type": "uint8", "internalType": "enum Gen0Reroll.RerollStatus"}, {"name": "finishTime", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "initialize", "type": "function", "inputs": [{"name": "_startTime", "type": "uint256", "internalType": "uint256"}, {"name": "_endTime", "type": "uint256", "internalType": "uint256"}, {"name": "_heroCoreAddress", "type": "address", "internalType": "address"}, {"name": "_statScienceAddress", "type": "address", "internalType": "address"}, {"name": "_flagStorageAddress", "type": "address", "internalType": "address"}, {"name": "_assistingAuction", "type": "address", "internalType": "address"}, {"name": "_rerollModifier", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setCrystal", "type": "function", "inputs": [{"name": "_index", "type": "uint8", "internalType": "uint8"}, {"name": "_address", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setRerollModifier", "type": "function", "inputs": [{"name": "_rerollModifier", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setStone", "type": "function", "inputs": [{"name": "_index", "type": "uint8", "internalType": "uint8"}, {"name": "_address", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setTimes", "type": "function", "inputs": [{"name": "_startTime", "type": "uint256", "internalType": "uint256"}, {"name": "_endTime", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "startRoll", "type": "function", "inputs": [{"name": "_heroId", "type": "uint256", "internalType": "uint256"}, {"name": "_choice", "type": "uint8", "internalType": "enum Gen0Reroll.RerollChoices"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "startTime", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "stones", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"}
]
"""     

class Gen0Reroll(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def campaign(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.campaign().call(block_identifier=block_identifier)

    def cancel_reroll(self, cred:Credentials, _hero_id:uint256) -> TxReceipt:
        tx = self.contract.functions.cancelReroll(_hero_id)
        return self.send_transaction(tx, cred)

    def crystals(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.crystals(a).call(block_identifier=block_identifier)

    def end_roll(self, cred:Credentials, _hero_id:uint256) -> TxReceipt:
        tx = self.contract.functions.endRoll(_hero_id)
        return self.send_transaction(tx, cred)

    def end_time(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.endTime().call(block_identifier=block_identifier)

    def hero_to_reroll_data(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> Tuple[uint256, uint256, uint8, tuple, uint8, uint256]:
        return self.contract.functions.heroToRerollData(a).call(block_identifier=block_identifier)

    def initialize(self, cred:Credentials, _start_time:uint256, _end_time:uint256, _hero_core_address:address, _stat_science_address:address, _flag_storage_address:address, _assisting_auction:address, _reroll_modifier:address) -> TxReceipt:
        tx = self.contract.functions.initialize(_start_time, _end_time, _hero_core_address, _stat_science_address, _flag_storage_address, _assisting_auction, _reroll_modifier)
        return self.send_transaction(tx, cred)

    def set_crystal(self, cred:Credentials, _index:uint8, _address:address) -> TxReceipt:
        tx = self.contract.functions.setCrystal(_index, _address)
        return self.send_transaction(tx, cred)

    def set_reroll_modifier(self, cred:Credentials, _reroll_modifier:address) -> TxReceipt:
        tx = self.contract.functions.setRerollModifier(_reroll_modifier)
        return self.send_transaction(tx, cred)

    def set_stone(self, cred:Credentials, _index:uint8, _address:address) -> TxReceipt:
        tx = self.contract.functions.setStone(_index, _address)
        return self.send_transaction(tx, cred)

    def set_times(self, cred:Credentials, _start_time:uint256, _end_time:uint256) -> TxReceipt:
        tx = self.contract.functions.setTimes(_start_time, _end_time)
        return self.send_transaction(tx, cred)

    def start_roll(self, cred:Credentials, _hero_id:uint256, _choice:uint8) -> TxReceipt:
        tx = self.contract.functions.startRoll(_hero_id, _choice)
        return self.send_transaction(tx, cred)

    def start_time(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.startTime().call(block_identifier=block_identifier)

    def stones(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.stones(a).call(block_identifier=block_identifier)



from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0x947873092dc57C1A70704033c41cB110f4462a8B",
    "sd": "0x0000000000000000000000000000000000000000"
}

ABI = """[
    {"name": "Claimed", "type": "event", "inputs": [{"name": "recipient", "type": "address", "indexed": false, "internalType": "address"}, {"name": "token", "type": "address", "indexed": false, "internalType": "address"}, {"name": "amount", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "Dropped", "type": "event", "inputs": [{"name": "recipient", "type": "address", "indexed": false, "internalType": "address"}, {"name": "token", "type": "address", "indexed": false, "internalType": "address"}, {"name": "amount", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "Initialized", "type": "event", "inputs": [{"name": "version", "type": "uint8", "indexed": false, "internalType": "uint8"}], "anonymous": false},
    {"name": "Withdrawal", "type": "event", "inputs": [{"name": "tokenAddress", "type": "address", "indexed": false, "internalType": "address"}, {"name": "recipient", "type": "address", "indexed": false, "internalType": "address"}, {"name": "amount", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "claimAirdrop", "type": "function", "inputs": [{"name": "_dropId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "deleteAirdrop", "type": "function", "inputs": [{"name": "player", "type": "address", "internalType": "address"}, {"name": "_dropId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "enabled", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "initialize", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setAirdrops", "type": "function", "inputs": [{"name": "_tokenAddress", "type": "address", "internalType": "address"}, {"name": "_recipients", "type": "address[]", "internalType": "address[]"}, {"name": "_amounts", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "_note", "type": "string", "internalType": "string"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "toggleEnabled", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "tokenToPendingAmount", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "viewAirdrops", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "tuple[]", "components": [{"name": "tokenAddress", "type": "address", "internalType": "address"}, {"name": "amount", "type": "uint256", "internalType": "uint256"}, {"name": "time", "type": "uint256", "internalType": "uint256"}, {"name": "note", "type": "string", "internalType": "string"}], "internalType": "struct AirdropClaim.Drop[]"}], "stateMutability": "view"},
    {"name": "viewUserAirdrops", "type": "function", "inputs": [{"name": "player", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "tuple[]", "components": [{"name": "tokenAddress", "type": "address", "internalType": "address"}, {"name": "amount", "type": "uint256", "internalType": "uint256"}, {"name": "time", "type": "uint256", "internalType": "uint256"}, {"name": "note", "type": "string", "internalType": "string"}], "internalType": "struct AirdropClaim.Drop[]"}], "stateMutability": "view"},
    {"name": "withdrawTokens", "type": "function", "inputs": [{"name": "_tokenAddress", "type": "address", "internalType": "address"}, {"name": "_recipient", "type": "address", "internalType": "address"}, {"name": "_amount", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"}
]
"""     

class AirdropClaim(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def claim_airdrop(self, cred:Credentials, _drop_id:uint256) -> TxReceipt:
        tx = self.contract.functions.claimAirdrop(_drop_id)
        return self.send_transaction(tx, cred)

    def delete_airdrop(self, cred:Credentials, player:address, _drop_id:uint256) -> TxReceipt:
        tx = self.contract.functions.deleteAirdrop(player, _drop_id)
        return self.send_transaction(tx, cred)

    def enabled(self, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.enabled().call(block_identifier=block_identifier)

    def initialize(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.initialize()
        return self.send_transaction(tx, cred)

    def set_airdrops(self, cred:Credentials, _token_address:address, _recipients:Sequence[address], _amounts:Sequence[uint256], _note:string) -> TxReceipt:
        tx = self.contract.functions.setAirdrops(_token_address, _recipients, _amounts, _note)
        return self.send_transaction(tx, cred)

    def toggle_enabled(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.toggleEnabled()
        return self.send_transaction(tx, cred)

    def token_to_pending_amount(self, a:address, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.tokenToPendingAmount(a).call(block_identifier=block_identifier)

    def view_airdrops(self, block_identifier:BlockIdentifier = 'latest') -> List[tuple]:
        return self.contract.functions.viewAirdrops().call(block_identifier=block_identifier)

    def view_user_airdrops(self, player:address, block_identifier:BlockIdentifier = 'latest') -> List[tuple]:
        return self.contract.functions.viewUserAirdrops(player).call(block_identifier=block_identifier)

    def withdraw_tokens(self, cred:Credentials, _token_address:address, _recipient:address, _amount:uint256) -> TxReceipt:
        tx = self.contract.functions.withdrawTokens(_token_address, _recipient, _amount)
        return self.send_transaction(tx, cred)



from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0xD507b6b299d9FC835a0Df92f718920D13fA49B47",
    "sd": "0xdbEE8C336B06f2d30a6d2bB3817a3Ae0E34f4900"
}

ABI = """[
    {"name": "AttunementCrystalAdded", "type": "event", "inputs": [{"name": "atunementItemAddress", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "Initialized", "type": "event", "inputs": [{"name": "version", "type": "uint8", "indexed": false, "internalType": "uint8"}], "anonymous": false},
    {"name": "LevelUp", "type": "event", "inputs": [{"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "heroId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "hero", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "summoningInfo", "type": "tuple", "components": [{"name": "summonedTime", "type": "uint256", "internalType": "uint256"}, {"name": "nextSummonTime", "type": "uint256", "internalType": "uint256"}, {"name": "summonerId", "type": "uint256", "internalType": "uint256"}, {"name": "assistantId", "type": "uint256", "internalType": "uint256"}, {"name": "summons", "type": "uint32", "internalType": "uint32"}, {"name": "maxSummons", "type": "uint32", "internalType": "uint32"}], "internalType": "struct IHeroTypes.SummoningInfo"}, {"name": "info", "type": "tuple", "components": [{"name": "statGenes", "type": "uint256", "internalType": "uint256"}, {"name": "visualGenes", "type": "uint256", "internalType": "uint256"}, {"name": "rarity", "type": "uint8", "internalType": "enum IHeroTypes.Rarity"}, {"name": "shiny", "type": "bool", "internalType": "bool"}, {"name": "generation", "type": "uint16", "internalType": "uint16"}, {"name": "firstName", "type": "uint32", "internalType": "uint32"}, {"name": "lastName", "type": "uint32", "internalType": "uint32"}, {"name": "shinyStyle", "type": "uint8", "internalType": "uint8"}, {"name": "class", "type": "uint8", "internalType": "uint8"}, {"name": "subClass", "type": "uint8", "internalType": "uint8"}], "internalType": "struct IHeroTypes.HeroInfo"}, {"name": "state", "type": "tuple", "components": [{"name": "staminaFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "hpFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "mpFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "level", "type": "uint16", "internalType": "uint16"}, {"name": "xp", "type": "uint64", "internalType": "uint64"}, {"name": "currentQuest", "type": "address", "internalType": "address"}, {"name": "sp", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "enum IHeroTypes.HeroStatus"}], "internalType": "struct IHeroTypes.HeroState"}, {"name": "stats", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hp", "type": "uint16", "internalType": "uint16"}, {"name": "mp", "type": "uint16", "internalType": "uint16"}, {"name": "stamina", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroStats"}, {"name": "primaryStatGrowth", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hpSm", "type": "uint16", "internalType": "uint16"}, {"name": "hpRg", "type": "uint16", "internalType": "uint16"}, {"name": "hpLg", "type": "uint16", "internalType": "uint16"}, {"name": "mpSm", "type": "uint16", "internalType": "uint16"}, {"name": "mpRg", "type": "uint16", "internalType": "uint16"}, {"name": "mpLg", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroStatGrowth"}, {"name": "secondaryStatGrowth", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hpSm", "type": "uint16", "internalType": "uint16"}, {"name": "hpRg", "type": "uint16", "internalType": "uint16"}, {"name": "hpLg", "type": "uint16", "internalType": "uint16"}, {"name": "mpSm", "type": "uint16", "internalType": "uint16"}, {"name": "mpRg", "type": "uint16", "internalType": "uint16"}, {"name": "mpLg", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroStatGrowth"}, {"name": "professions", "type": "tuple", "components": [{"name": "mining", "type": "uint16", "internalType": "uint16"}, {"name": "gardening", "type": "uint16", "internalType": "uint16"}, {"name": "foraging", "type": "uint16", "internalType": "uint16"}, {"name": "fishing", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroProfessions"}], "indexed": false, "internalType": "struct IHeroTypes.Hero"}, {"name": "oldHero", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "summoningInfo", "type": "tuple", "components": [{"name": "summonedTime", "type": "uint256", "internalType": "uint256"}, {"name": "nextSummonTime", "type": "uint256", "internalType": "uint256"}, {"name": "summonerId", "type": "uint256", "internalType": "uint256"}, {"name": "assistantId", "type": "uint256", "internalType": "uint256"}, {"name": "summons", "type": "uint32", "internalType": "uint32"}, {"name": "maxSummons", "type": "uint32", "internalType": "uint32"}], "internalType": "struct IHeroTypes.SummoningInfo"}, {"name": "info", "type": "tuple", "components": [{"name": "statGenes", "type": "uint256", "internalType": "uint256"}, {"name": "visualGenes", "type": "uint256", "internalType": "uint256"}, {"name": "rarity", "type": "uint8", "internalType": "enum IHeroTypes.Rarity"}, {"name": "shiny", "type": "bool", "internalType": "bool"}, {"name": "generation", "type": "uint16", "internalType": "uint16"}, {"name": "firstName", "type": "uint32", "internalType": "uint32"}, {"name": "lastName", "type": "uint32", "internalType": "uint32"}, {"name": "shinyStyle", "type": "uint8", "internalType": "uint8"}, {"name": "class", "type": "uint8", "internalType": "uint8"}, {"name": "subClass", "type": "uint8", "internalType": "uint8"}], "internalType": "struct IHeroTypes.HeroInfo"}, {"name": "state", "type": "tuple", "components": [{"name": "staminaFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "hpFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "mpFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "level", "type": "uint16", "internalType": "uint16"}, {"name": "xp", "type": "uint64", "internalType": "uint64"}, {"name": "currentQuest", "type": "address", "internalType": "address"}, {"name": "sp", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "enum IHeroTypes.HeroStatus"}], "internalType": "struct IHeroTypes.HeroState"}, {"name": "stats", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hp", "type": "uint16", "internalType": "uint16"}, {"name": "mp", "type": "uint16", "internalType": "uint16"}, {"name": "stamina", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroStats"}, {"name": "primaryStatGrowth", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hpSm", "type": "uint16", "internalType": "uint16"}, {"name": "hpRg", "type": "uint16", "internalType": "uint16"}, {"name": "hpLg", "type": "uint16", "internalType": "uint16"}, {"name": "mpSm", "type": "uint16", "internalType": "uint16"}, {"name": "mpRg", "type": "uint16", "internalType": "uint16"}, {"name": "mpLg", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroStatGrowth"}, {"name": "secondaryStatGrowth", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hpSm", "type": "uint16", "internalType": "uint16"}, {"name": "hpRg", "type": "uint16", "internalType": "uint16"}, {"name": "hpLg", "type": "uint16", "internalType": "uint16"}, {"name": "mpSm", "type": "uint16", "internalType": "uint16"}, {"name": "mpRg", "type": "uint16", "internalType": "uint16"}, {"name": "mpLg", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroStatGrowth"}, {"name": "professions", "type": "tuple", "components": [{"name": "mining", "type": "uint16", "internalType": "uint16"}, {"name": "gardening", "type": "uint16", "internalType": "uint16"}, {"name": "foraging", "type": "uint16", "internalType": "uint16"}, {"name": "fishing", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IHeroTypes.HeroProfessions"}], "indexed": false, "internalType": "struct IHeroTypes.Hero"}], "anonymous": false},
    {"name": "MeditationBegun", "type": "event", "inputs": [{"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "heroId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "meditationId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "primaryStat", "type": "uint8", "indexed": false, "internalType": "uint8"}, {"name": "secondaryStat", "type": "uint8", "indexed": false, "internalType": "uint8"}, {"name": "tertiaryStat", "type": "uint8", "indexed": false, "internalType": "uint8"}, {"name": "attunementCrystal", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "MeditationCompleted", "type": "event", "inputs": [{"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "heroId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "meditationId", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "Paused", "type": "event", "inputs": [{"name": "account", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "StatUp", "type": "event", "inputs": [{"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "heroId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "stat", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "increase", "type": "uint8", "indexed": false, "internalType": "uint8"}, {"name": "updateType", "type": "uint8", "indexed": false, "internalType": "enum MeditationCircle.UpdateType"}], "anonymous": false},
    {"name": "Unpaused", "type": "event", "inputs": [{"name": "account", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "_getRequiredRunes", "type": "function", "inputs": [{"name": "_level", "type": "uint16", "internalType": "uint16"}], "outputs": [{"name": "", "type": "uint16[10]", "internalType": "uint16[10]"}], "stateMutability": "pure"},
    {"name": "activeAttunementCrystals", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "addAttunementCrystal", "type": "function", "inputs": [{"name": "_address", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "adminRemove", "type": "function", "inputs": [{"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "completeMeditation", "type": "function", "inputs": [{"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "getActiveMeditations", "type": "function", "inputs": [{"name": "_address", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "tuple[]", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "heroId", "type": "uint256", "internalType": "uint256"}, {"name": "primaryStat", "type": "uint8", "internalType": "uint8"}, {"name": "secondaryStat", "type": "uint8", "internalType": "uint8"}, {"name": "tertiaryStat", "type": "uint8", "internalType": "uint8"}, {"name": "attunementCrystal", "type": "address", "internalType": "address"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "status", "type": "uint8", "internalType": "uint8"}], "internalType": "struct MeditationCircle.Meditation[]"}], "stateMutability": "view"},
    {"name": "getHeroMeditation", "type": "function", "inputs": [{"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "heroId", "type": "uint256", "internalType": "uint256"}, {"name": "primaryStat", "type": "uint8", "internalType": "uint8"}, {"name": "secondaryStat", "type": "uint8", "internalType": "uint8"}, {"name": "tertiaryStat", "type": "uint8", "internalType": "uint8"}, {"name": "attunementCrystal", "type": "address", "internalType": "address"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "status", "type": "uint8", "internalType": "uint8"}], "internalType": "struct MeditationCircle.Meditation"}], "stateMutability": "view"},
    {"name": "getMeditation", "type": "function", "inputs": [{"name": "_id", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "heroId", "type": "uint256", "internalType": "uint256"}, {"name": "primaryStat", "type": "uint8", "internalType": "uint8"}, {"name": "secondaryStat", "type": "uint8", "internalType": "uint8"}, {"name": "tertiaryStat", "type": "uint8", "internalType": "uint8"}, {"name": "attunementCrystal", "type": "address", "internalType": "address"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "status", "type": "uint8", "internalType": "uint8"}], "internalType": "struct MeditationCircle.Meditation"}], "stateMutability": "view"},
    {"name": "heroToMeditation", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "initialize", "type": "function", "inputs": [{"name": "_heroCoreAddress", "type": "address", "internalType": "address"}, {"name": "_statScienceAddress", "type": "address", "internalType": "address"}, {"name": "_jewelTokenAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "jewelToken", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "contract IJewelToken"}], "stateMutability": "view"},
    {"name": "pause", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "paused", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "profileActiveMeditations", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}, {"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "heroId", "type": "uint256", "internalType": "uint256"}, {"name": "primaryStat", "type": "uint8", "internalType": "uint8"}, {"name": "secondaryStat", "type": "uint8", "internalType": "uint8"}, {"name": "tertiaryStat", "type": "uint8", "internalType": "uint8"}, {"name": "attunementCrystal", "type": "address", "internalType": "address"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "status", "type": "uint8", "internalType": "uint8"}], "stateMutability": "view"},
    {"name": "removeAttunementCrystal", "type": "function", "inputs": [{"name": "_address", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "runes", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "address", "internalType": "contract IInventoryItem"}], "stateMutability": "view"},
    {"name": "setFees", "type": "function", "inputs": [{"name": "_feeAddresses", "type": "address[]", "internalType": "address[]"}, {"name": "_feePercents", "type": "uint256[]", "internalType": "uint256[]"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setRune", "type": "function", "inputs": [{"name": "_index", "type": "uint8", "internalType": "uint8"}, {"name": "_address", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setStatScienceAddress", "type": "function", "inputs": [{"name": "_statScienceAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "startMeditation", "type": "function", "inputs": [{"name": "_heroId", "type": "uint256", "internalType": "uint256"}, {"name": "_primaryStat", "type": "uint8", "internalType": "uint8"}, {"name": "_secondaryStat", "type": "uint8", "internalType": "uint8"}, {"name": "_tertiaryStat", "type": "uint8", "internalType": "uint8"}, {"name": "_attunementCrystal", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "unpause", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"}
]
"""     

class MeditationCircle(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def _get_required_runes(self, _level:uint16, block_identifier:BlockIdentifier = 'latest') -> List[uint16]:
        return self.contract.functions._getRequiredRunes(_level).call(block_identifier=block_identifier)

    def active_attunement_crystals(self, a:address, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.activeAttunementCrystals(a).call(block_identifier=block_identifier)

    def add_attunement_crystal(self, cred:Credentials, _address:address) -> TxReceipt:
        tx = self.contract.functions.addAttunementCrystal(_address)
        return self.send_transaction(tx, cred)

    def admin_remove(self, cred:Credentials, _hero_id:uint256) -> TxReceipt:
        tx = self.contract.functions.adminRemove(_hero_id)
        return self.send_transaction(tx, cred)

    def complete_meditation(self, cred:Credentials, _hero_id:uint256) -> TxReceipt:
        tx = self.contract.functions.completeMeditation(_hero_id)
        return self.send_transaction(tx, cred)

    def get_active_meditations(self, _address:address, block_identifier:BlockIdentifier = 'latest') -> List[tuple]:
        return self.contract.functions.getActiveMeditations(_address).call(block_identifier=block_identifier)

    def get_hero_meditation(self, _hero_id:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getHeroMeditation(_hero_id).call(block_identifier=block_identifier)

    def get_meditation(self, _id:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getMeditation(_id).call(block_identifier=block_identifier)

    def hero_to_meditation(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.heroToMeditation(a).call(block_identifier=block_identifier)

    def initialize(self, cred:Credentials, _hero_core_address:address, _stat_science_address:address, _jewel_token_address:address) -> TxReceipt:
        tx = self.contract.functions.initialize(_hero_core_address, _stat_science_address, _jewel_token_address)
        return self.send_transaction(tx, cred)

    def jewel_token(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.jewelToken().call(block_identifier=block_identifier)

    def pause(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.pause()
        return self.send_transaction(tx, cred)

    def paused(self, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.paused().call(block_identifier=block_identifier)

    def profile_active_meditations(self, a:address, b:uint256, block_identifier:BlockIdentifier = 'latest') -> Tuple[uint256, address, uint256, uint8, uint8, uint8, address, uint256, uint8]:
        return self.contract.functions.profileActiveMeditations(a, b).call(block_identifier=block_identifier)

    def remove_attunement_crystal(self, cred:Credentials, _address:address) -> TxReceipt:
        tx = self.contract.functions.removeAttunementCrystal(_address)
        return self.send_transaction(tx, cred)

    def runes(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.runes(a).call(block_identifier=block_identifier)

    def set_fees(self, cred:Credentials, _fee_addresses:Sequence[address], _fee_percents:Sequence[uint256]) -> TxReceipt:
        tx = self.contract.functions.setFees(_fee_addresses, _fee_percents)
        return self.send_transaction(tx, cred)

    def set_rune(self, cred:Credentials, _index:uint8, _address:address) -> TxReceipt:
        tx = self.contract.functions.setRune(_index, _address)
        return self.send_transaction(tx, cred)

    def set_stat_science_address(self, cred:Credentials, _stat_science_address:address) -> TxReceipt:
        tx = self.contract.functions.setStatScienceAddress(_stat_science_address)
        return self.send_transaction(tx, cred)

    def start_meditation(self, cred:Credentials, _hero_id:uint256, _primary_stat:uint8, _secondary_stat:uint8, _tertiary_stat:uint8, _attunement_crystal:address) -> TxReceipt:
        tx = self.contract.functions.startMeditation(_hero_id, _primary_stat, _secondary_stat, _tertiary_stat, _attunement_crystal)
        return self.send_transaction(tx, cred)

    def unpause(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.unpause()
        return self.send_transaction(tx, cred)



from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0x184223A0921B58F5a0ddFD6448d8b2715EcC87a7",
    "sd": "0x0000000000000000000000000000000000000000"
}

ABI = """[
    {"name": "Approval", "type": "event", "inputs": [{"name": "owner", "type": "address", "indexed": true, "internalType": "address"}, {"name": "spender", "type": "address", "indexed": true, "internalType": "address"}, {"name": "value", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "Initialized", "type": "event", "inputs": [{"name": "version", "type": "uint8", "indexed": false, "internalType": "uint8"}], "anonymous": false},
    {"name": "Paused", "type": "event", "inputs": [{"name": "account", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "Transfer", "type": "event", "inputs": [{"name": "from", "type": "address", "indexed": true, "internalType": "address"}, {"name": "to", "type": "address", "indexed": true, "internalType": "address"}, {"name": "value", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "Unpaused", "type": "event", "inputs": [{"name": "account", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "allowance", "type": "function", "inputs": [{"name": "owner", "type": "address", "internalType": "address"}, {"name": "spender", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "approve", "type": "function", "inputs": [{"name": "spender", "type": "address", "internalType": "address"}, {"name": "amount", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "nonpayable"},
    {"name": "balanceOf", "type": "function", "inputs": [{"name": "account", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "burn", "type": "function", "inputs": [{"name": "amount", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "burnFrom", "type": "function", "inputs": [{"name": "account", "type": "address", "internalType": "address"}, {"name": "amount", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "burnable", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "decimals", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint8", "internalType": "uint8"}], "stateMutability": "view"},
    {"name": "decreaseAllowance", "type": "function", "inputs": [{"name": "spender", "type": "address", "internalType": "address"}, {"name": "subtractedValue", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "nonpayable"},
    {"name": "increaseAllowance", "type": "function", "inputs": [{"name": "spender", "type": "address", "internalType": "address"}, {"name": "addedValue", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "nonpayable"},
    {"name": "initialize", "type": "function", "inputs": [{"name": "_name", "type": "string", "internalType": "string"}, {"name": "_symbol", "type": "string", "internalType": "string"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "mint", "type": "function", "inputs": [{"name": "to", "type": "address", "internalType": "address"}, {"name": "amount", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "name", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "string", "internalType": "string"}], "stateMutability": "view"},
    {"name": "pause", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "paused", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "symbol", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "string", "internalType": "string"}], "stateMutability": "view"},
    {"name": "toggleBurnable", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "togglePause", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "totalSupply", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "transfer", "type": "function", "inputs": [{"name": "to", "type": "address", "internalType": "address"}, {"name": "amount", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "nonpayable"},
    {"name": "transferFrom", "type": "function", "inputs": [{"name": "from", "type": "address", "internalType": "address"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "amount", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "nonpayable"},
    {"name": "unpause", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"}
]
"""     

class BirthdayCake(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def allowance(self, owner:address, spender:address, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.allowance(owner, spender).call(block_identifier=block_identifier)

    def approve(self, cred:Credentials, spender:address, amount:uint256) -> TxReceipt:
        tx = self.contract.functions.approve(spender, amount)
        return self.send_transaction(tx, cred)

    def balance_of(self, account:address, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.balanceOf(account).call(block_identifier=block_identifier)

    def burn(self, cred:Credentials, amount:uint256) -> TxReceipt:
        tx = self.contract.functions.burn(amount)
        return self.send_transaction(tx, cred)

    def burn_from(self, cred:Credentials, account:address, amount:uint256) -> TxReceipt:
        tx = self.contract.functions.burnFrom(account, amount)
        return self.send_transaction(tx, cred)

    def burnable(self, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.burnable().call(block_identifier=block_identifier)

    def decimals(self, block_identifier:BlockIdentifier = 'latest') -> uint8:
        return self.contract.functions.decimals().call(block_identifier=block_identifier)

    def decrease_allowance(self, cred:Credentials, spender:address, subtracted_value:uint256) -> TxReceipt:
        tx = self.contract.functions.decreaseAllowance(spender, subtracted_value)
        return self.send_transaction(tx, cred)

    def increase_allowance(self, cred:Credentials, spender:address, added_value:uint256) -> TxReceipt:
        tx = self.contract.functions.increaseAllowance(spender, added_value)
        return self.send_transaction(tx, cred)

    def initialize(self, cred:Credentials, _name:string, _symbol:string) -> TxReceipt:
        tx = self.contract.functions.initialize(_name, _symbol)
        return self.send_transaction(tx, cred)

    def mint(self, cred:Credentials, to:address, amount:uint256) -> TxReceipt:
        tx = self.contract.functions.mint(to, amount)
        return self.send_transaction(tx, cred)

    def name(self, block_identifier:BlockIdentifier = 'latest') -> string:
        return self.contract.functions.name().call(block_identifier=block_identifier)

    def pause(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.pause()
        return self.send_transaction(tx, cred)

    def paused(self, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.paused().call(block_identifier=block_identifier)

    def symbol(self, block_identifier:BlockIdentifier = 'latest') -> string:
        return self.contract.functions.symbol().call(block_identifier=block_identifier)

    def toggle_burnable(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.toggleBurnable()
        return self.send_transaction(tx, cred)

    def toggle_pause(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.togglePause()
        return self.send_transaction(tx, cred)

    def total_supply(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.totalSupply().call(block_identifier=block_identifier)

    def transfer(self, cred:Credentials, to:address, amount:uint256) -> TxReceipt:
        tx = self.contract.functions.transfer(to, amount)
        return self.send_transaction(tx, cred)

    def transfer_from(self, cred:Credentials, _from:address, to:address, amount:uint256) -> TxReceipt:
        tx = self.contract.functions.transferFrom(_from, to, amount)
        return self.send_transaction(tx, cred)

    def unpause(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.unpause()
        return self.send_transaction(tx, cred)


#! /usr/bin/env python
from web3 import Web3
from web3.middleware.geth_poa import geth_poa_middleware

from .abi_contract_wrapper import ABIContractWrapper

from .solidity_types import *
from typing import Dict, Tuple, Union, Optional, Any

DEFAULT_TIMEOUT = 30
DEFAULT_MAX_GAS = 50
DEFAULT_MAX_PRIORITY_GAS = 3

W3_INSTANCES: Dict[str, Web3] = {}

class ABIMultiContractWrapper(ABIContractWrapper):
    def __init__(self, 
                 abi:str,
                 rpc:str,
                 max_gas_gwei:float=DEFAULT_MAX_GAS,
                 max_priority_gwei:float=DEFAULT_MAX_PRIORITY_GAS):
        self.rpc = rpc
        self.abi = abi
        self.nonces: Dict[address, int] = {}
        self.timeout = DEFAULT_TIMEOUT

        # This one superclass may be used by many contracts, who don't all need to
        # create separate Web3 instances. Just use one per RPC
        global W3_INSTANCES
        w3 = W3_INSTANCES.get(self.rpc, None)
        if not w3:
            w3 = Web3(Web3.HTTPProvider(self.rpc))
            w3.middleware_onion.inject(geth_poa_middleware, layer=0)
            W3_INSTANCES[self.rpc] = w3
        self.w3 = w3

        self.max_gas_wei = self.w3.to_wei(max_gas_gwei, 'gwei')
        self.max_priority_wei = self.w3.to_wei(max_priority_gwei, 'gwei')




from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0x501CdC4ef10b63219704Bf6aDb785dfccb06deE2",
    "sd": "0x6d5B86EaC9097EA4a94B2b69Cd4854678B89f839"
}

ABI = """[
    {"name": "ERC1155Received", "type": "event", "inputs": [{"name": "_srcChainId", "type": "uint16", "internalType": "uint16", "indexed": false}, {"name": "_srcAddress", "type": "bytes", "internalType": "bytes", "indexed": false}, {"name": "receiver", "type": "address", "internalType": "address", "indexed": false}, {"name": "item", "type": "address", "internalType": "address", "indexed": false}, {"name": "amount", "type": "uint256", "internalType": "uint256", "indexed": false}, {"name": "id", "type": "uint256", "internalType": "uint256", "indexed": false}], "anonymous": false},
    {"name": "Initialized", "type": "event", "inputs": [{"name": "version", "type": "uint8", "internalType": "uint8", "indexed": false}], "anonymous": false},
    {"name": "ItemMapped", "type": "event", "inputs": [{"name": "identifier", "type": "bytes32", "internalType": "bytes32", "indexed": false}, {"name": "contractAddress", "type": "address", "internalType": "address", "indexed": false}], "anonymous": false},
    {"name": "ItemReceived", "type": "event", "inputs": [{"name": "_srcChainId", "type": "uint16", "internalType": "uint16", "indexed": false}, {"name": "_srcAddress", "type": "bytes", "internalType": "bytes", "indexed": false}, {"name": "receiver", "type": "address", "internalType": "address", "indexed": false}, {"name": "item", "type": "address", "internalType": "address", "indexed": false}, {"name": "amount", "type": "uint256", "internalType": "uint256", "indexed": false}], "anonymous": false},
    {"name": "MessageFailed", "type": "event", "inputs": [{"name": "_srcChainId", "type": "uint16", "internalType": "uint16", "indexed": false}, {"name": "_srcAddress", "type": "bytes", "internalType": "bytes", "indexed": false}, {"name": "_nonce", "type": "uint64", "internalType": "uint64", "indexed": false}, {"name": "_payload", "type": "bytes", "internalType": "bytes", "indexed": false}], "anonymous": false},
    {"name": "OwnershipTransferred", "type": "event", "inputs": [{"name": "previousOwner", "type": "address", "internalType": "address", "indexed": true}, {"name": "newOwner", "type": "address", "internalType": "address", "indexed": true}], "anonymous": false},
    {"name": "Paused", "type": "event", "inputs": [{"name": "account", "type": "address", "internalType": "address", "indexed": false}], "anonymous": false},
    {"name": "SetTrustedRemote", "type": "event", "inputs": [{"name": "_srcChainId", "type": "uint16", "internalType": "uint16", "indexed": false}, {"name": "_srcAddress", "type": "bytes", "internalType": "bytes", "indexed": false}], "anonymous": false},
    {"name": "Unpaused", "type": "event", "inputs": [{"name": "account", "type": "address", "internalType": "address", "indexed": false}], "anonymous": false},
    {"name": "FUNCTION_TYPE_SEND", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "NO_EXTRA_GAS", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "__NonblockingLzAppUpgradeable_init", "type": "function", "inputs": [{"name": "_lzEndpoint", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "__NonblockingLzAppUpgradeable_init_unchained", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "estimateFeeSendERC1155", "type": "function", "inputs": [{"name": "_dstChainId", "type": "uint16", "internalType": "uint16"}, {"name": "_receiver", "type": "address", "internalType": "address"}, {"name": "_item", "type": "address", "internalType": "address"}, {"name": "_amount", "type": "uint256", "internalType": "uint256"}, {"name": "_id", "type": "uint256", "internalType": "uint256"}, {"name": "_useZro", "type": "bool", "internalType": "bool"}, {"name": "_adapterParams", "type": "bytes", "internalType": "bytes"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "estimateFeeSendERC1155", "type": "function", "inputs": [{"name": "_dstChainId", "type": "uint16", "internalType": "uint16"}, {"name": "_receiver", "type": "address", "internalType": "address"}, {"name": "_item", "type": "address", "internalType": "address"}, {"name": "_amount", "type": "uint256", "internalType": "uint256"}, {"name": "_id", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "estimateFeeSendERC20", "type": "function", "inputs": [{"name": "_dstChainId", "type": "uint16", "internalType": "uint16"}, {"name": "_receiver", "type": "address", "internalType": "address"}, {"name": "_item", "type": "address", "internalType": "address"}, {"name": "_amount", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "estimateFeeSendERC20", "type": "function", "inputs": [{"name": "_dstChainId", "type": "uint16", "internalType": "uint16"}, {"name": "_receiver", "type": "address", "internalType": "address"}, {"name": "_item", "type": "address", "internalType": "address"}, {"name": "_amount", "type": "uint256", "internalType": "uint256"}, {"name": "_useZro", "type": "bool", "internalType": "bool"}, {"name": "_adapterParams", "type": "bytes", "internalType": "bytes"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "failedMessages", "type": "function", "inputs": [{"name": "", "type": "uint16", "internalType": "uint16"}, {"name": "", "type": "bytes", "internalType": "bytes"}, {"name": "", "type": "uint64", "internalType": "uint64"}], "outputs": [{"name": "", "type": "bytes32", "internalType": "bytes32"}], "stateMutability": "view"},
    {"name": "forceResumeReceive", "type": "function", "inputs": [{"name": "_srcChainId", "type": "uint16", "internalType": "uint16"}, {"name": "_srcAddress", "type": "bytes", "internalType": "bytes"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "getConfig", "type": "function", "inputs": [{"name": "_version", "type": "uint16", "internalType": "uint16"}, {"name": "_chainId", "type": "uint16", "internalType": "uint16"}, {"name": "", "type": "address", "internalType": "address"}, {"name": "_configType", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "bytes", "internalType": "bytes"}], "stateMutability": "view"},
    {"name": "getGasLimit", "type": "function", "inputs": [{"name": "_adapterParams", "type": "bytes", "internalType": "bytes"}], "outputs": [{"name": "gasLimit", "type": "uint256", "internalType": "uint256"}], "stateMutability": "pure"},
    {"name": "initialize", "type": "function", "inputs": [{"name": "_lzEndpoint", "type": "address", "internalType": "address"}, {"name": "_itemMinter", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "isTrustedRemote", "type": "function", "inputs": [{"name": "_srcChainId", "type": "uint16", "internalType": "uint16"}, {"name": "_srcAddress", "type": "bytes", "internalType": "bytes"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "itemAddresses", "type": "function", "inputs": [{"name": "", "type": "bytes32", "internalType": "bytes32"}], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "itemIdentifiers", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "bytes32", "internalType": "bytes32"}], "stateMutability": "view"},
    {"name": "lzEndpoint", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "contract ILayerZeroEndpointUpgradeable"}], "stateMutability": "view"},
    {"name": "lzReceive", "type": "function", "inputs": [{"name": "_srcChainId", "type": "uint16", "internalType": "uint16"}, {"name": "_srcAddress", "type": "bytes", "internalType": "bytes"}, {"name": "_nonce", "type": "uint64", "internalType": "uint64"}, {"name": "_payload", "type": "bytes", "internalType": "bytes"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "mapItem", "type": "function", "inputs": [{"name": "_identifier", "type": "string", "internalType": "string"}, {"name": "_contractAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "minDstGasLookup", "type": "function", "inputs": [{"name": "", "type": "uint16", "internalType": "uint16"}, {"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "nonblockingLzReceive", "type": "function", "inputs": [{"name": "_srcChainId", "type": "uint16", "internalType": "uint16"}, {"name": "_srcAddress", "type": "bytes", "internalType": "bytes"}, {"name": "_nonce", "type": "uint64", "internalType": "uint64"}, {"name": "_payload", "type": "bytes", "internalType": "bytes"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "owner", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "pause", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "paused", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "renounceOwnership", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "retryMessage", "type": "function", "inputs": [{"name": "_srcChainId", "type": "uint16", "internalType": "uint16"}, {"name": "_srcAddress", "type": "bytes", "internalType": "bytes"}, {"name": "_nonce", "type": "uint64", "internalType": "uint64"}, {"name": "_payload", "type": "bytes", "internalType": "bytes"}], "outputs": [], "stateMutability": "payable"},
    {"name": "sendERC1155", "type": "function", "inputs": [{"name": "_dstChainId", "type": "uint16", "internalType": "uint16"}, {"name": "_receiver", "type": "address", "internalType": "address"}, {"name": "_item", "type": "address", "internalType": "address"}, {"name": "_amount", "type": "uint256", "internalType": "uint256"}, {"name": "_id", "type": "uint256", "internalType": "uint256"}, {"name": "_zroPaymentAddress", "type": "address", "internalType": "address"}, {"name": "_adapterParams", "type": "bytes", "internalType": "bytes"}], "outputs": [], "stateMutability": "payable"},
    {"name": "sendERC1155", "type": "function", "inputs": [{"name": "_dstChainId", "type": "uint16", "internalType": "uint16"}, {"name": "_receiver", "type": "address", "internalType": "address"}, {"name": "_item", "type": "address", "internalType": "address"}, {"name": "_amount", "type": "uint256", "internalType": "uint256"}, {"name": "_id", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "payable"},
    {"name": "sendERC20", "type": "function", "inputs": [{"name": "_dstChainId", "type": "uint16", "internalType": "uint16"}, {"name": "_receiver", "type": "address", "internalType": "address"}, {"name": "_item", "type": "address", "internalType": "address"}, {"name": "_amount", "type": "uint256", "internalType": "uint256"}, {"name": "_zroPaymentAddress", "type": "address", "internalType": "address"}, {"name": "_adapterParams", "type": "bytes", "internalType": "bytes"}], "outputs": [], "stateMutability": "payable"},
    {"name": "sendERC20", "type": "function", "inputs": [{"name": "_dstChainId", "type": "uint16", "internalType": "uint16"}, {"name": "_receiver", "type": "address", "internalType": "address"}, {"name": "_item", "type": "address", "internalType": "address"}, {"name": "_amount", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "payable"},
    {"name": "setConfig", "type": "function", "inputs": [{"name": "_version", "type": "uint16", "internalType": "uint16"}, {"name": "_chainId", "type": "uint16", "internalType": "uint16"}, {"name": "_configType", "type": "uint256", "internalType": "uint256"}, {"name": "_config", "type": "bytes", "internalType": "bytes"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setMinDstGasLookup", "type": "function", "inputs": [{"name": "_dstChainId", "type": "uint16", "internalType": "uint16"}, {"name": "_type", "type": "uint256", "internalType": "uint256"}, {"name": "_dstGasAmount", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setReceiveVersion", "type": "function", "inputs": [{"name": "_version", "type": "uint16", "internalType": "uint16"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setSendVersion", "type": "function", "inputs": [{"name": "_version", "type": "uint16", "internalType": "uint16"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setTrustedRemote", "type": "function", "inputs": [{"name": "_srcChainId", "type": "uint16", "internalType": "uint16"}, {"name": "_srcAddress", "type": "bytes", "internalType": "bytes"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setUseCustomAdapterParams", "type": "function", "inputs": [{"name": "_useCustomAdapterParams", "type": "bool", "internalType": "bool"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "transferOwnership", "type": "function", "inputs": [{"name": "newOwner", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "trustedRemoteLookup", "type": "function", "inputs": [{"name": "", "type": "uint16", "internalType": "uint16"}], "outputs": [{"name": "", "type": "bytes", "internalType": "bytes"}], "stateMutability": "view"},
    {"name": "unpause", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "useCustomAdapterParams", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"}
]
"""     

class ItemBridge(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def function_type_send(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.FUNCTION_TYPE_SEND().call(block_identifier=block_identifier)

    def no_extra_gas(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.NO_EXTRA_GAS().call(block_identifier=block_identifier)

    def ___nonblocking_lz_app_upgradeable_init(self, cred:Credentials, _lz_endpoint:address) -> TxReceipt:
        tx = self.contract.functions.__NonblockingLzAppUpgradeable_init(_lz_endpoint)
        return self.send_transaction(tx, cred)

    def ___nonblocking_lz_app_upgradeable_init_unchained(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.__NonblockingLzAppUpgradeable_init_unchained()
        return self.send_transaction(tx, cred)

    def estimate_fee_send_erc1155(self, _dst_chain_id:uint16, _receiver:address, _item:address, _amount:uint256, _id:uint256, _use_zro:bool, _adapter_params:bytes, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.estimateFeeSendERC1155(_dst_chain_id, _receiver, _item, _amount, _id, _use_zro, _adapter_params).call(block_identifier=block_identifier)

    def estimate_fee_send_erc1155(self, _dst_chain_id:uint16, _receiver:address, _item:address, _amount:uint256, _id:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.estimateFeeSendERC1155(_dst_chain_id, _receiver, _item, _amount, _id).call(block_identifier=block_identifier)

    def estimate_fee_send_erc20(self, _dst_chain_id:uint16, _receiver:address, _item:address, _amount:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.estimateFeeSendERC20(_dst_chain_id, _receiver, _item, _amount).call(block_identifier=block_identifier)

    def estimate_fee_send_erc20(self, _dst_chain_id:uint16, _receiver:address, _item:address, _amount:uint256, _use_zro:bool, _adapter_params:bytes, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.estimateFeeSendERC20(_dst_chain_id, _receiver, _item, _amount, _use_zro, _adapter_params).call(block_identifier=block_identifier)

    def failed_messages(self, a:uint16, b:bytes, c:uint64, block_identifier:BlockIdentifier = 'latest') -> bytes32:
        return self.contract.functions.failedMessages(a, b, c).call(block_identifier=block_identifier)

    def force_resume_receive(self, cred:Credentials, _src_chain_id:uint16, _src_address:bytes) -> TxReceipt:
        tx = self.contract.functions.forceResumeReceive(_src_chain_id, _src_address)
        return self.send_transaction(tx, cred)

    def get_config(self, _version:uint16, _chain_id:uint16, a:address, _config_type:uint256, block_identifier:BlockIdentifier = 'latest') -> bytes:
        return self.contract.functions.getConfig(_version, _chain_id, a, _config_type).call(block_identifier=block_identifier)

    def get_gas_limit(self, _adapter_params:bytes, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getGasLimit(_adapter_params).call(block_identifier=block_identifier)

    def initialize(self, cred:Credentials, _lz_endpoint:address, _item_minter:address) -> TxReceipt:
        tx = self.contract.functions.initialize(_lz_endpoint, _item_minter)
        return self.send_transaction(tx, cred)

    def is_trusted_remote(self, _src_chain_id:uint16, _src_address:bytes, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.isTrustedRemote(_src_chain_id, _src_address).call(block_identifier=block_identifier)

    def item_addresses(self, a:bytes32, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.itemAddresses(a).call(block_identifier=block_identifier)

    def item_identifiers(self, a:address, block_identifier:BlockIdentifier = 'latest') -> bytes32:
        return self.contract.functions.itemIdentifiers(a).call(block_identifier=block_identifier)

    def lz_endpoint(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.lzEndpoint().call(block_identifier=block_identifier)

    def lz_receive(self, cred:Credentials, _src_chain_id:uint16, _src_address:bytes, _nonce:uint64, _payload:bytes) -> TxReceipt:
        tx = self.contract.functions.lzReceive(_src_chain_id, _src_address, _nonce, _payload)
        return self.send_transaction(tx, cred)

    def map_item(self, cred:Credentials, _identifier:string, _contract_address:address) -> TxReceipt:
        tx = self.contract.functions.mapItem(_identifier, _contract_address)
        return self.send_transaction(tx, cred)

    def min_dst_gas_lookup(self, a:uint16, b:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.minDstGasLookup(a, b).call(block_identifier=block_identifier)

    def nonblocking_lz_receive(self, cred:Credentials, _src_chain_id:uint16, _src_address:bytes, _nonce:uint64, _payload:bytes) -> TxReceipt:
        tx = self.contract.functions.nonblockingLzReceive(_src_chain_id, _src_address, _nonce, _payload)
        return self.send_transaction(tx, cred)

    def owner(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.owner().call(block_identifier=block_identifier)

    def pause(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.pause()
        return self.send_transaction(tx, cred)

    def paused(self, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.paused().call(block_identifier=block_identifier)

    def renounce_ownership(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.renounceOwnership()
        return self.send_transaction(tx, cred)

    def retry_message(self, cred:Credentials, _src_chain_id:uint16, _src_address:bytes, _nonce:uint64, _payload:bytes) -> TxReceipt:
        tx = self.contract.functions.retryMessage(_src_chain_id, _src_address, _nonce, _payload)
        return self.send_transaction(tx, cred)

    def send_erc1155(self, cred:Credentials, _dst_chain_id:uint16, _receiver:address, _item:address, _amount:uint256, _id:uint256, _zro_payment_address:address, _adapter_params:bytes) -> TxReceipt:
        tx = self.contract.functions.sendERC1155(_dst_chain_id, _receiver, _item, _amount, _id, _zro_payment_address, _adapter_params)
        return self.send_transaction(tx, cred)

    def send_erc1155(self, cred:Credentials, _dst_chain_id:uint16, _receiver:address, _item:address, _amount:uint256, _id:uint256) -> TxReceipt:
        tx = self.contract.functions.sendERC1155(_dst_chain_id, _receiver, _item, _amount, _id)
        return self.send_transaction(tx, cred)

    def send_erc20(self, cred:Credentials, _dst_chain_id:uint16, _receiver:address, _item:address, _amount:uint256, _zro_payment_address:address, _adapter_params:bytes) -> TxReceipt:
        tx = self.contract.functions.sendERC20(_dst_chain_id, _receiver, _item, _amount, _zro_payment_address, _adapter_params)
        return self.send_transaction(tx, cred)

    def send_erc20(self, cred:Credentials, _dst_chain_id:uint16, _receiver:address, _item:address, _amount:uint256) -> TxReceipt:
        tx = self.contract.functions.sendERC20(_dst_chain_id, _receiver, _item, _amount)
        return self.send_transaction(tx, cred)

    def set_config(self, cred:Credentials, _version:uint16, _chain_id:uint16, _config_type:uint256, _config:bytes) -> TxReceipt:
        tx = self.contract.functions.setConfig(_version, _chain_id, _config_type, _config)
        return self.send_transaction(tx, cred)

    def set_min_dst_gas_lookup(self, cred:Credentials, _dst_chain_id:uint16, _type:uint256, _dst_gas_amount:uint256) -> TxReceipt:
        tx = self.contract.functions.setMinDstGasLookup(_dst_chain_id, _type, _dst_gas_amount)
        return self.send_transaction(tx, cred)

    def set_receive_version(self, cred:Credentials, _version:uint16) -> TxReceipt:
        tx = self.contract.functions.setReceiveVersion(_version)
        return self.send_transaction(tx, cred)

    def set_send_version(self, cred:Credentials, _version:uint16) -> TxReceipt:
        tx = self.contract.functions.setSendVersion(_version)
        return self.send_transaction(tx, cred)

    def set_trusted_remote(self, cred:Credentials, _src_chain_id:uint16, _src_address:bytes) -> TxReceipt:
        tx = self.contract.functions.setTrustedRemote(_src_chain_id, _src_address)
        return self.send_transaction(tx, cred)

    def set_use_custom_adapter_params(self, cred:Credentials, _use_custom_adapter_params:bool) -> TxReceipt:
        tx = self.contract.functions.setUseCustomAdapterParams(_use_custom_adapter_params)
        return self.send_transaction(tx, cred)

    def transfer_ownership(self, cred:Credentials, new_owner:address) -> TxReceipt:
        tx = self.contract.functions.transferOwnership(new_owner)
        return self.send_transaction(tx, cred)

    def trusted_remote_lookup(self, a:uint16, block_identifier:BlockIdentifier = 'latest') -> bytes:
        return self.contract.functions.trustedRemoteLookup(a).call(block_identifier=block_identifier)

    def unpause(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.unpause()
        return self.send_transaction(tx, cred)

    def use_custom_adapter_params(self, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.useCustomAdapterParams().call(block_identifier=block_identifier)



from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0x68f6C64786cfCb35108986041D1009c9d27bde22",
    "sd": "0x13cE9c99E8E2fcDe1632adA7B69b2eCf5BE8ED45"
}

ABI = """[
    {"name": "CrystalAirdrop", "type": "event", "inputs": [{"name": "owner", "type": "address", "indexed": true, "internalType": "address"}, {"name": "crystalId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "createdBlock", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "CrystalOpen", "type": "event", "inputs": [{"name": "owner", "type": "address", "indexed": true, "internalType": "address"}, {"name": "crystalId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "heroId", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "CrystalSummoned", "type": "event", "inputs": [{"name": "crystalId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "owner", "type": "address", "indexed": true, "internalType": "address"}, {"name": "summonerId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "assistantId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "generation", "type": "uint16", "indexed": false, "internalType": "uint16"}, {"name": "createdBlock", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "summonerTears", "type": "uint8", "indexed": false, "internalType": "uint8"}, {"name": "assistantTears", "type": "uint8", "indexed": false, "internalType": "uint8"}, {"name": "enhancementStone", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "EnhancementStoneAdded", "type": "event", "inputs": [{"name": "atunementItemAddress", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "Initialized", "type": "event", "inputs": [{"name": "version", "type": "uint8", "indexed": false, "internalType": "uint8"}], "anonymous": false},
    {"name": "Paused", "type": "event", "inputs": [{"name": "account", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "RoleAdminChanged", "type": "event", "inputs": [{"name": "role", "type": "bytes32", "indexed": true, "internalType": "bytes32"}, {"name": "previousAdminRole", "type": "bytes32", "indexed": true, "internalType": "bytes32"}, {"name": "newAdminRole", "type": "bytes32", "indexed": true, "internalType": "bytes32"}], "anonymous": false},
    {"name": "RoleGranted", "type": "event", "inputs": [{"name": "role", "type": "bytes32", "indexed": true, "internalType": "bytes32"}, {"name": "account", "type": "address", "indexed": true, "internalType": "address"}, {"name": "sender", "type": "address", "indexed": true, "internalType": "address"}], "anonymous": false},
    {"name": "RoleRevoked", "type": "event", "inputs": [{"name": "role", "type": "bytes32", "indexed": true, "internalType": "bytes32"}, {"name": "account", "type": "address", "indexed": true, "internalType": "address"}, {"name": "sender", "type": "address", "indexed": true, "internalType": "address"}], "anonymous": false},
    {"name": "Unpaused", "type": "event", "inputs": [{"name": "account", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "CRYSTAL_MODERATOR_ROLE", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bytes32", "internalType": "bytes32"}], "stateMutability": "view"},
    {"name": "DEFAULT_ADMIN_ROLE", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bytes32", "internalType": "bytes32"}], "stateMutability": "view"},
    {"name": "MODERATOR_ROLE", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bytes32", "internalType": "bytes32"}], "stateMutability": "view"},
    {"name": "airdropCrystal", "type": "function", "inputs": [{"name": "_recipient", "type": "address", "internalType": "address"}, {"name": "_isShiny", "type": "bool", "internalType": "bool"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "createCrystal", "type": "function", "inputs": [{"name": "_owner", "type": "address", "internalType": "address"}, {"name": "_summonerId", "type": "uint256", "internalType": "uint256"}, {"name": "_assistantId", "type": "uint256", "internalType": "uint256"}, {"name": "_generation", "type": "uint16", "internalType": "uint16"}, {"name": "_summonerBonusTears", "type": "uint8", "internalType": "uint8"}, {"name": "_assistantBonusTears", "type": "uint8", "internalType": "uint8"}, {"name": "_enhancementStone", "type": "address", "internalType": "address"}, {"name": "_maxSummons", "type": "uint32", "internalType": "uint32"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "getCrystal", "type": "function", "inputs": [{"name": "_crystalId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "internalType": "struct HeroCrystal", "components": [{"name": "owner", "type": "address", "internalType": "address"}, {"name": "summonerId", "type": "uint256", "internalType": "uint256"}, {"name": "assistantId", "type": "uint256", "internalType": "uint256"}, {"name": "generation", "type": "uint16", "internalType": "uint16"}, {"name": "createdBlock", "type": "uint256", "internalType": "uint256"}, {"name": "heroId", "type": "uint256", "internalType": "uint256"}, {"name": "summonerTears", "type": "uint8", "internalType": "uint8"}, {"name": "assistantTears", "type": "uint8", "internalType": "uint8"}, {"name": "enhancementStone", "type": "address", "internalType": "address"}, {"name": "maxSummons", "type": "uint32", "internalType": "uint32"}, {"name": "firstName", "type": "uint32", "internalType": "uint32"}, {"name": "lastName", "type": "uint32", "internalType": "uint32"}, {"name": "shinyStyle", "type": "uint8", "internalType": "uint8"}]}], "stateMutability": "view"},
    {"name": "getRoleAdmin", "type": "function", "inputs": [{"name": "role", "type": "bytes32", "internalType": "bytes32"}], "outputs": [{"name": "", "type": "bytes32", "internalType": "bytes32"}], "stateMutability": "view"},
    {"name": "getUserCrystals", "type": "function", "inputs": [{"name": "_address", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "uint256[]", "internalType": "uint256[]"}], "stateMutability": "view"},
    {"name": "grantRole", "type": "function", "inputs": [{"name": "role", "type": "bytes32", "internalType": "bytes32"}, {"name": "account", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "hasRole", "type": "function", "inputs": [{"name": "role", "type": "bytes32", "internalType": "bytes32"}, {"name": "account", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "initialize", "type": "function", "inputs": [{"name": "_heroCoreAddress", "type": "address", "internalType": "address"}, {"name": "_statScienceAddress", "type": "address", "internalType": "address"}, {"name": "_randomGeneratorAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "newSummonCooldown", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "open", "type": "function", "inputs": [{"name": "_crystalId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "nonpayable"},
    {"name": "pause", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "paused", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "renounceRole", "type": "function", "inputs": [{"name": "role", "type": "bytes32", "internalType": "bytes32"}, {"name": "account", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "revokeRole", "type": "function", "inputs": [{"name": "role", "type": "bytes32", "internalType": "bytes32"}, {"name": "account", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setHeroCore", "type": "function", "inputs": [{"name": "_heroCoreAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setStatScience", "type": "function", "inputs": [{"name": "_statScienceAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setWaitBlocks", "type": "function", "inputs": [{"name": "_waitBlocks", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "supportsInterface", "type": "function", "inputs": [{"name": "interfaceId", "type": "bytes4", "internalType": "bytes4"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "totalCrystals", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "unpause", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "waitBlocks", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"}
]
"""     

class CrystalCore(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def airdrop_crystal(self, cred:Credentials, _recipient:address, _is_shiny:bool) -> TxReceipt:
        tx = self.contract.functions.airdropCrystal(_recipient, _is_shiny)
        return self.send_transaction(tx, cred)

    def create_crystal(self, cred:Credentials, _owner:address, _summoner_id:uint256, _assistant_id:uint256, _generation:uint16, _summoner_bonus_tears:uint8, _assistant_bonus_tears:uint8, _enhancement_stone:address, _max_summons:uint32) -> TxReceipt:
        tx = self.contract.functions.createCrystal(_owner, _summoner_id, _assistant_id, _generation, _summoner_bonus_tears, _assistant_bonus_tears, _enhancement_stone, _max_summons)
        return self.send_transaction(tx, cred)

    def get_crystal(self, _crystal_id:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getCrystal(_crystal_id).call(block_identifier=block_identifier)

    def get_user_crystals(self, _address:address, block_identifier:BlockIdentifier = 'latest') -> List[uint256]:
        return self.contract.functions.getUserCrystals(_address).call(block_identifier=block_identifier)

    def initialize(self, cred:Credentials, _hero_core_address:address, _stat_science_address:address, _random_generator_address:address) -> TxReceipt:
        tx = self.contract.functions.initialize(_hero_core_address, _stat_science_address, _random_generator_address)
        return self.send_transaction(tx, cred)

    def new_summon_cooldown(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.newSummonCooldown().call(block_identifier=block_identifier)

    def open(self, cred:Credentials, _crystal_id:uint256) -> TxReceipt:
        tx = self.contract.functions.open(_crystal_id)
        return self.send_transaction(tx, cred)

    def pause(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.pause()
        return self.send_transaction(tx, cred)

    def paused(self, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.paused().call(block_identifier=block_identifier)

    def set_hero_core(self, cred:Credentials, _hero_core_address:address) -> TxReceipt:
        tx = self.contract.functions.setHeroCore(_hero_core_address)
        return self.send_transaction(tx, cred)

    def set_stat_science(self, cred:Credentials, _stat_science_address:address) -> TxReceipt:
        tx = self.contract.functions.setStatScience(_stat_science_address)
        return self.send_transaction(tx, cred)

    def set_wait_blocks(self, cred:Credentials, _wait_blocks:uint256) -> TxReceipt:
        tx = self.contract.functions.setWaitBlocks(_wait_blocks)
        return self.send_transaction(tx, cred)

    def supports_interface(self, interface_id:bytes4, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.supportsInterface(interface_id).call(block_identifier=block_identifier)

    def total_crystals(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.totalCrystals().call(block_identifier=block_identifier)

    def unpause(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.unpause()
        return self.send_transaction(tx, cred)

    def wait_blocks(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.waitBlocks().call(block_identifier=block_identifier)



from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0x794C07912474351b3134E6D6B3B7b3b4A07cbAAa",
    "sd": "0x36fAE766e51f17F8218C735f58426E293498Db2B"
}

ABI = """[
    {"type": "constructor", "inputs": [{"name": "_feeToSetter", "type": "address", "internalType": "address"}], "stateMutability": "nonpayable"},
    {"name": "PairCreated", "type": "event", "inputs": [{"name": "token0", "type": "address", "indexed": true, "internalType": "address"}, {"name": "token1", "type": "address", "indexed": true, "internalType": "address"}, {"name": "pair", "type": "address", "indexed": false, "internalType": "address"}, {"name": "", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "allPairs", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "allPairsLength", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "createPair", "type": "function", "inputs": [{"name": "tokenA", "type": "address", "internalType": "address"}, {"name": "tokenB", "type": "address", "internalType": "address"}], "outputs": [{"name": "pair", "type": "address", "internalType": "address"}], "stateMutability": "nonpayable"},
    {"name": "feeTo", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "feeToSetter", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "getPair", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}, {"name": "", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "pairCodeHash", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bytes32", "internalType": "bytes32"}], "stateMutability": "pure"},
    {"name": "setFeeTo", "type": "function", "inputs": [{"name": "_feeTo", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setFeeToSetter", "type": "function", "inputs": [{"name": "_feeToSetter", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"}
]
"""     

class UniswapV2Factory(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def all_pairs(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.allPairs(a).call(block_identifier=block_identifier)

    def all_pairs_length(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.allPairsLength().call(block_identifier=block_identifier)

    def create_pair(self, cred:Credentials, token_a:address, token_b:address) -> TxReceipt:
        tx = self.contract.functions.createPair(token_a, token_b)
        return self.send_transaction(tx, cred)

    def fee_to(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.feeTo().call(block_identifier=block_identifier)

    def fee_to_setter(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.feeToSetter().call(block_identifier=block_identifier)

    def get_pair(self, a:address, b:address, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.getPair(a, b).call(block_identifier=block_identifier)

    def pair_code_hash(self, block_identifier:BlockIdentifier = 'latest') -> bytes32:
        return self.contract.functions.pairCodeHash().call(block_identifier=block_identifier)

    def set_fee_to(self, cred:Credentials, _fee_to:address) -> TxReceipt:
        tx = self.contract.functions.setFeeTo(_fee_to)
        return self.send_transaction(tx, cred)

    def set_fee_to_setter(self, cred:Credentials, _fee_to_setter:address) -> TxReceipt:
        tx = self.contract.functions.setFeeToSetter(_fee_to_setter)
        return self.send_transaction(tx, cred)